import urllib
import urlparse

from django.shortcuts import redirect

from django.conf import settings

def hosted_login_url(hosted_login_token):
    return 'https://%s.recurly.com/account/%s' % (
        settings.RECURLY_SUBDOMAIN,
        hosted_login_token,
    )

def hosted_payment_page_url(plan_code, account_code, data=None):
    if data is None:
        data = {}

    return 'https://%s.recurly.com/subscribe/%s/%d?%s' % (
        settings.RECURLY_SUBDOMAIN,
        plan_code,
        account_code,
        urllib.urlencode(data),
    )

def safe_redirect(url, fallback="/"):
    netloc = urlparse.urlparse(url or "")[1]
    
    if not url:
        safe_url = fallback
    # Don't redirect the user to a different host
    elif netloc and netloc != request.get_host():
        safe_url = fallback
    else:
        safe_url = url
    
    return redirect(safe_url)


#!/usr/bin/env python

# Based largely on:
# http://ericholscher.com/blog/2009/jun/29/enable-setuppy-test-your-django-apps/

import os, sys
os.environ['DJANGO_SETTINGS_MODULE'] = 'settings'

test_dir = os.path.dirname(__file__)
sys.path.insert(0, test_dir)

from django.test.utils import get_runner
from django.conf import settings

def runtests():
    test_runner_class = get_runner(settings)
    test_runner = test_runner_class(verbosity=1, interactive=True)
    failures = test_runner.run_tests(["django_recurly"])
    sys.exit(failures)

if __name__ == '__main__':
    runtests()



import random
import string
import iso8601
import pytz

from django.contrib.auth.models import User

def random_string(length=32):
    return ''.join(random.choice(string.letters + string.digits) for i in xrange(length))

def modelify(data, model, key_prefix="", remove_empty=False, date_fields=[]):
    fields = set(field.name for field in model._meta.fields)
    fields_by_name = dict((field.name, field) for field in model._meta.fields)
    fields.discard("id")
    
    if "user" in fields and data.get("username", None):
        try:
            data["user"] = User.objects.get(username=data["username"])
        except User.DoesNotExist:
            # A user may not exist if there account has been deleted
            data["user"] = None
    
    for k, v in data.items():
        if isinstance(v, dict):
            data.update(modelify(v, model, key_prefix=k+"_"))
    
    out = {}
    for k, v in data.items():
        if not k.startswith(key_prefix):
            k = key_prefix + k
        
        if k in fields:
            if k.endswith("_at") or k in date_fields:
                v = iso8601.parse_date(v).astimezone(tz=pytz.utc) if v else None
            
            # Always assume fields with limited choices shoudl be lower case
            if v and fields_by_name[k].choices:
                v = v.lower()
            
            if v or not remove_empty:
                out[str(k)] = v
    
    return out


# Hook up the signals to the relevant actions

from django_recurly import signals
from django_recurly import models

def new(data, **kwargs):
    """Create the account and the subscription
    
    We do these at the same time (rather than using 
    the new_account signal) to avoid concurrency problems.
    """
    models.Account.handle_notification(data)

def update(data, **kwargs):
    """Update a subscription and account"""
    models.Account.handle_notification(data)

def payment(data, **kwargs):
    models.Payment.handle_notification(data)

signals.new_subscription_notification.connect(new)
signals.updated_subscription_notification.connect(update)
signals.expired_subscription_notification.connect(update)
signals.canceled_subscription_notification.connect(update)
signals.renewed_subscription_notification.connect(update)
signals.reactivated_account_notification.connect(update)

signals.canceled_account_notification.connect(update)
signals.billing_info_updated_notification.connect(update)

signals.successful_payment_notification.connect(payment)
signals.failed_payment_notification.connect(payment)
signals.successful_refund_notification.connect(payment)
signals.void_payment_notification.connect(payment)



import unittest
import datetime

from django.test import TestCase

from django_recurly import views
from django_recurly.tests.base import BaseTest, RequestFactory
from django_recurly.models import *

rf = RequestFactory()

class PushNotificationViewTest(BaseTest):
    
    def test_all(self):
        
        for name, xml in self.push_notifications.items():
            if "_refund_" in name or "_payment_" in name:
                # This is a payment notification, for which we need an account to exist
                request = rf.post("/junk", self.push_notifications["new_subscription_notification-ok"], content_type="text/xml")
                response = views.push_notifications(request)
            
            signal, t = name.split("-")
            request = rf.post("/junk", xml, content_type="text/xml")
            
            self.resetSignals()
            response = views.push_notifications(request)
            self.assertEqual(response.status_code, 200)
            self.assertSignal(signal)
        
    
    def test_new_subscription(self):
        request = rf.post("/junk", self.push_notifications["new_subscription_notification-ok"], content_type="text/xml")
        response = views.push_notifications(request)
        self.assertEqual(response.status_code, 200)
        
        account = Account.objects.latest()
        
        self.assertEqual(Account.objects.count(), 1)
        self.assertEqual(account.user.username, "verena")
        self.assertEqual(account.first_name, "Verena")
        self.assertEqual(account.company_name, "Company, Inc.")
        self.assertEqual(account.email, "verena@test.com")
        self.assertEqual(account.account_code, "verena@test.com")
        
        subscription = account.get_current_subscription()
        self.assertEqual(Subscription.objects.count(), 1)
        self.assertEqual(subscription.plan_code, "bronze")
        self.assertEqual(subscription.plan_version, 2)
        self.assertEqual(subscription.state, "active")
        self.assertEqual(subscription.quantity, 2)
        self.assertEqual(subscription.total_amount_in_cents, 2000)
        self.assertEqual(subscription.activated_at, datetime.datetime(2009, 11, 22, 21, 10, 38)) # Phew, its in UTC now :)
        
        
        
        
    



from django.test import TestCase

from django_recurly.helpers import api
from django_recurly.tests.base import BaseTest, RequestFactory
from django_recurly.models import *
from django_recurly import views

rf = RequestFactory()

class GetSubscribeUrlTest(BaseTest):
    
    def test_encoding(self):
        html = api.get_change_plan_form(plan_code="myplan")
        
        self.assertTrue('<form action="/change-plan/" method="POST">' in html, html)
        self.assertTrue('input type="hidden" name="plan_code" value="myplan">' in html, html)
    



from django.template.loader import render_to_string

from django_recurly.models import Account, Subscription

def get_change_plan_form(plan_code):
    return render_to_string("django_recurly/change_plan_form.html", {
        "plan_code": plan_code,
    })


from django.db import models
from django.db.models import Q
from django.contrib.auth.models import User
from datetime import datetime
import pytz

from timezones.fields import LocalizedDateTimeField

from django_recurly.utilities import random_string, modelify
from django_recurly import signals
from django_recurly.client import get_client

SUBSCRIPTION_STATES = (
    ("active", "Active"),         # Active and everything is fine
    ("canceled", "Canceled"),   # Still active, but will not be renewed
    ("expired", "Expired"),       # Did not renews, or was forcibly expired early
)

__all__ = ("Account", "Subscription", "User", "Payment")

class CurrentAccountManager(models.Manager):
    def get_query_set(self):
        return super(CurrentAccountManager, self).get_query_set().filter(canceled=False)
    

class CurrentSubscriptionManager(models.Manager):
    def get_query_set(self):
        return super(CurrentSubscriptionManager, self).get_query_set().filter(Q(super_subscription=True) | Q(state__in=("active", "canceled")))
    

class Account(models.Model):
    account_code = models.CharField(max_length=32, unique=True)
    user = models.ForeignKey(User, related_name="recurly_account", blank=True, null=True, on_delete=models.SET_NULL)
    created_at = LocalizedDateTimeField(default=datetime.now())
    email = models.CharField(max_length=100)
    first_name = models.CharField(max_length=100)
    last_name = models.CharField(max_length=100)
    company_name = models.CharField(max_length=100, blank=True, null=True)
    canceled = models.BooleanField(default=False)
    hosted_login_token = models.CharField(max_length=32, blank=True, null=True)
    
    objects = models.Manager()
    current = CurrentAccountManager()
    
    class Meta:
        ordering = ["-id"]
        get_latest_by = "id"
    
    def get_subscriptions(self):
        """Get all subscriptions for this Account
        
        An account may have multiple subscriptions in cases
        where old subscriptions expired.
        
        If you need the current subscription, consider 
        using get_current_subscription()
        """
        return self.subscription_set.all()
    
    def get_current_subscription(self):
        try:
            return Subscription.current.filter(account=self).latest()
        except Subscription.DoesNotExist:
            return None
    
    def fetch_hosted_login_token(self):
        raise NotImplemented("Well, it's not. Sorry.")
    
    @classmethod
    def get_current(class_, user):
        return class_.current.filter(user=user).latest()
    
    @classmethod
    def handle_notification(class_, data):
        """Update/create an account and it's associated subscription using data from Recurly"""
        # First get/create the account
        account_data = modelify(data.get("account"), Account)
        
        account, created = class_.objects.get_or_create(
            account_code=account_data["account_code"],
            defaults=account_data
        )
        
        if not created:
            for k, v in account_data.items():
                setattr(account, k, v)
            account.save()
        
        # Now get/create the subscription
        if not data.get("subscription"):
            return account, None
        
        subscription_data = modelify(data.get("subscription"), Subscription)
        subscription = account.get_current_subscription()
        
        
        if not subscription:
            # Not found, create it
            subscription = Subscription.objects.create(account=account, **subscription_data)
            
            was_current = False
            now_current = subscription.is_current()
            
        else:
            was_current = subscription.is_current()
            
            # Found, update it
            for k, v in subscription_data.items():
                setattr(subscription, k, v)
            subscription.save()
            
            now_current = subscription.is_current()
        
        # Send account closed/opened signals
        if was_current and not now_current:
            signals.account_closed.send(sender=account, account=account, subscription=subscription)
        elif not was_current and now_current:
            signals.account_opened.send(sender=account, account=account, subscription=subscription)
        
        return account, subscription
    
    @classmethod
    def create_fake(class_, user, plan_code):
        """Create a fake account for a user
        
        Creates an account which exists in Django, but not in Recurly. This can be 
        handy for testing, or the occasional time when you just want to give someone 
        a paid subscription without any fuss.
        """
        
        account = class_.objects.create(
            account_code="fake_%s" % random_string(32 - 5),
            user=user,
            email=user.email,
            first_name=user.first_name,
            last_name=user.last_name,
            canceled=False
        )
        
        subscription = Subscription.objects.create(
            account=account,
            plan_code=plan_code
        )
        
        signals.account_opened.send(sender=account, account=account, subscription=subscription)
        
        return account, subscription
    
    def get_transactions(self):
        client = get_client()
        response = client.accounts.transactions(account_code=self.account_code)
        return response["transaction"]
        

class Subscription(models.Model):
    account = models.ForeignKey(Account)
    plan_code = models.CharField(max_length=100)
    plan_version = models.IntegerField(default=1)
    state = models.CharField(max_length=20, default="active", choices=SUBSCRIPTION_STATES)
    quantity = models.IntegerField(default=1)
    total_amount_in_cents = models.IntegerField(blank=True, null=True) # Not always in cents!
    activated_at = LocalizedDateTimeField(blank=True, null=True)
    canceled_at = LocalizedDateTimeField(blank=True, null=True)
    expires_at = LocalizedDateTimeField(blank=True, null=True)
    current_period_started_at = LocalizedDateTimeField(blank=True, null=True)
    current_period_ends_at = LocalizedDateTimeField(blank=True, null=True)
    trial_started_at = LocalizedDateTimeField(blank=True, null=True)
    trial_ends_at = LocalizedDateTimeField(blank=True, null=True)
    
    super_subscription = models.BooleanField(default=False)
    
    objects = models.Manager()
    current = CurrentSubscriptionManager()
    
    class Meta:
        ordering = ["-id"]
        get_latest_by = "id"
    
    def is_current(self):
        """Is this subscription current (i.e. not expired and good to be used)
        
        Note that 'canceled' accounts are actually still 'current', as 
        'canceled' just indicates they they will not renew after the 
        current billing period (at which point Recurly will tell us that 
        they are 'expired')
        """
        
        return bool(Subscription.current.filter(pk=self.pk).count())
    
    def is_trial(self):
        if self.super_subscription:
            return False
        
        if not self.trial_started_at or not self.trial_ends_at:
            return False # No trial dates, so not a trial
        
        now = datetime.now(tz=pytz.utc)
        if self.trial_started_at <= now and self.trial_ends_at > now:
            return True
        else:
            return False
    
    def change_plan(self, plan_code):
        """Change this subscription to the specified plan_code.
        
        This will call the Recurly API.
        """
        client = get_client()
        
        update_data = {
            "timeframe": "now",
            "plan_code": plan_code
        }
        
        client.accounts.subscription.update(account_code=self.account.account_code, data=update_data)
        self.plan_code = plan_code
        self.save()
    
    def terminate(self, refund="none"):
        """Terminate the subscription
        
        refund may be one of:
            - "none" : No refund, subscription is just expired
            - "partial" : Give a prorated refund
            - "full" : Provide a full refund of the most recent charge
        """
        client = get_client()
        
        client.accounts.subscription.delete(account_code=self.account.account_code, refund=refund)
    
    def cancel(self):
        """Cancel the subscription, it will expire at the end of the current billing cycle"""
        client = get_client()
        
        client.accounts.subscription.delete(account_code=self.account.account_code)
    

class Payment(models.Model):
    
    ACTION_CHOICES = (
        ("purchase", "Purchase"),
        ("credit", "Credit"),
    )
    
    STATUS_CHOICES = (
        ("success", "Success"),
        ("declined", "Declined"),
        ("void", "Void"),
    )
    
    account = models.ForeignKey(Account, blank=True, null=True)
    transaction_id = models.CharField(max_length=40)
    invoice_id = models.CharField(max_length=40, blank=True, null=True)
    action = models.CharField(max_length=10, choices=ACTION_CHOICES)
    date = LocalizedDateTimeField(blank=True, null=True)
    amount_in_cents = models.IntegerField(blank=True, null=True) # Not always in cents!
    status = models.CharField(max_length=10, choices=STATUS_CHOICES)
    message = models.CharField(max_length=250)
    
    class Meta:
        ordering = ["-id"]
        get_latest_by = "id"
    
    @classmethod
    def handle_notification(class_, data):
        transaction = data.get("transaction")
        account_code = data["account"]["account_code"]
        
        transaction_data = modelify(transaction, class_, remove_empty=True, date_fields=["date"])
        
        transaction_data["transaction_id"] = transaction["id"]
        transaction_data["account"] = Account.objects.get(account_code=account_code)
        
        payment, created = class_.objects.get_or_create(
            transaction_id=transaction["id"],
            defaults=transaction_data
        )
        
        return payment
    











import functools

from django.conf import settings
from django.http import HttpResponse, HttpResponseBadRequest, \
    HttpResponseForbidden
from django.utils.crypto import constant_time_compare

def recurly_basic_authentication(fn):
    @functools.wraps(fn)
    def wrapper(request, *args, **kwargs):
        authentication = getattr(settings, 'RECURLY_HTTP_AUTHENTICATION', None)

        # If the user has not setup settings.RECURLY_HTTP_AUTHENTICATION then
        # we trust they are doing it at the web server level.
        if authentication is None:
            return fn(request, *args, **kwargs)

        try:
            method, auth = request.META['HTTP_AUTHORIZATION'].split(' ', 1)
        except KeyError:
            response = HttpResponse()
            response.status_code = 401
            response['WWW-Authenticate'] = 'Basic realm="Restricted"'
            return response

        try:
            if method.lower() != 'basic':
                raise ValueError()

            if not constant_time_compare(auth.strip().decode('base64'), authentication):
                return HttpResponseForbidden()
        except Exception:
            return HttpResponseBadRequest()

        return fn(request, *args, **kwargs)
    return wrapper



"""Template tags for working work Recurly's hosted payment pages"""

from django import template
from django.template import Library, Node, Variable, loader
from django.template.context import Context

from django_recurly.helpers.hosted import get_subscribe_url

register = template.Library()

@register.simple_tag
def subscribe_url(user, plan_code, quantity=1):
    return get_subscribe_url(user, plan_code, quantity)




import weakref
import glob
import os.path
import base64

from django.test import TestCase
from django.test.client import Client
from django.conf import settings
from django.core.handlers.wsgi import WSGIRequest

from django_recurly import signals
from django_recurly.models import *

class BaseTest(TestCase):
    def setUp(self):
        super(BaseTest, self).setUp()
        self._signals = set([])
        self._setUpSignals()
        self.setUpData()
        
        self.user = User.objects.create(username="verena", email="moo@cow.com")
    
    def tearDown(self):
        super(BaseTest, self).tearDown()
        self._tearDownSignals()
    
    def _setUpSignals(self):
        def reg(k, signal):
            signal.connect(lambda sender, **kwargs: self._receiveSignal(k, signal, sender, **kwargs), dispatch_uid="unittest_uid", weak=False)
        
        for k in signals.__dict__:
            signal = signals.__dict__[k]
            if hasattr(signal, "providing_args"):
                reg(k, signal)
    
    def _tearDownSignals(self):
        for k in signals.__dict__:
            signal = signals.__dict__[k]
            if hasattr(signal, "providing_args"):
                signal.disconnect(dispatch_uid="unittest_uid", weak=False)
    
    def _receiveSignal(self, signal_key, signal_object, sender, **kwargs):
        self._signals.add(signal_key)
    
    def assertSignal(self, signal):
        self.assertTrue(signal in self._signals, "Signal '%s' was never sent" % signal)
    
    def resetSignals(self):
        self._signals = set([])
    
    def assertNoSignal(self, signal):
        self.assertFalse(signal in self._signals, "Signal '%s' was sent" % signal)
    
    def setUpData(self):
        xml_dir = os.path.abspath(os.path.dirname(__file__)) + "/data/push_notifications/*/*"
        xml_files = glob.glob(xml_dir)
        
        self.push_notifications = {}
        for xml_file in xml_files:
            f = open(xml_file, "r")
            xml = f.read()
            f.close()
            
            name = xml_file.split("/")[-1]
            
            self.push_notifications[name] = xml
    
    def parse_xml(self, xml):
        from django_recurly.client import get_client
        
        client = get_client()
        client.parse_notification(xml)
        
        return client.response
    

class RequestFactory(Client):
    # Used to generate request objects.
    def request(self, **request):
        credentials = base64.encodestring(settings.RECURLY_HTTP_AUTHENTICATION).strip()
        auth_string = 'Basic %s' % credentials
        
        environ = {
            'HTTP_COOKIE': self.cookies,
            'PATH_INFO': '/',
            'QUERY_STRING': '',
            'REQUEST_METHOD': 'GET',
            'SCRIPT_NAME': '',
            'SERVER_NAME': 'testserver',
            'SERVER_PORT': 80,
            'SERVER_PROTOCOL': 'HTTP/1.1',
            'HTTP_AUTHORIZATION': auth_string,
        }
        environ.update(self.defaults)
        environ.update(request)
        return WSGIRequest(environ)
    



# http://docs.recurly.com/integration/push-notifications

from django.dispatch import Signal

## General use signals ##

# Fired when a user gains a valid (i.e. current) account
account_opened = Signal(providing_args=('account','subscription'))

# Fired when a user's valid (i.e. current) account stops being valid (i.e. is 'expired')
account_closed = Signal(providing_args=('account','subscription'))

## Notifications from recurly ##

# Accounts
new_account_notification = Signal(providing_args=('data',))
canceled_account_notification = Signal(providing_args=('data',))
billing_info_updated_notification = Signal(providing_args=('data',))

# Subscriptions
new_subscription_notification = Signal(providing_args=('data',))
updated_subscription_notification = Signal(providing_args=('data',))
expired_subscription_notification = Signal(providing_args=('data',))
canceled_subscription_notification = Signal(providing_args=('data',))
renewed_subscription_notification = Signal(providing_args=('data',))
reactivated_account_notification = Signal(providing_args=('data',))

# Payments
successful_payment_notification = Signal(providing_args=('data',))
failed_payment_notification = Signal(providing_args=('data',))
successful_refund_notification = Signal(providing_args=('data',))
void_payment_notification = Signal(providing_args=('data',))



from django.conf.urls.defaults import *
from django.views.generic.simple import direct_to_template

from django_recurly.views import push_notifications, change_plan, account

urlpatterns = patterns("",
    url(r"^recurly-notification/$", push_notifications, name="recurly_notification"),
    url(r"^change-plan/$", change_plan, name="change_plan"),
    url(r"^account/$", account, name="account"),
)



from django_recurly.tests.views import *
from django_recurly.tests.models import *
from django_recurly.tests.helpers import *
from django_recurly.tests.helpers.hosted import *
from django_recurly.tests.helpers.api import *


# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'Payment'
        db.create_table('django_recurly_payment', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('account', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['django_recurly.Account'], null=True, blank=True)),
            ('transaction_id', self.gf('django.db.models.fields.CharField')(max_length=40)),
            ('invoice_id', self.gf('django.db.models.fields.CharField')(max_length=40, null=True, blank=True)),
            ('action', self.gf('django.db.models.fields.CharField')(max_length=10)),
            ('date', self.gf('timezones.fields.LocalizedDateTimeField')(null=True, blank=True)),
            ('amount_in_cents', self.gf('django.db.models.fields.IntegerField')(null=True, blank=True)),
            ('status', self.gf('django.db.models.fields.CharField')(max_length=10)),
            ('message', self.gf('django.db.models.fields.CharField')(max_length=250)),
        ))
        db.send_create_signal('django_recurly', ['Payment'])

        # Changing field 'Subscription.total_amount_in_cents'
        db.alter_column('django_recurly_subscription', 'total_amount_in_cents', self.gf('django.db.models.fields.IntegerField')(null=True))


    def backwards(self, orm):
        
        # Deleting model 'Payment'
        db.delete_table('django_recurly_payment')

        # Changing field 'Subscription.total_amount_in_cents'
        db.alter_column('django_recurly_subscription', 'total_amount_in_cents', self.gf('django.db.models.fields.DecimalField')(null=True, max_digits=8, decimal_places=2))


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'django_recurly.account': {
            'Meta': {'ordering': "['-id']", 'object_name': 'Account'},
            'account_code': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '32'}),
            'canceled': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'company_name': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'created_at': ('timezones.fields.LocalizedDateTimeField', [], {'default': 'datetime.datetime(2011, 10, 5, 22, 11, 15, 437027)'}),
            'email': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'hosted_login_token': ('django.db.models.fields.CharField', [], {'max_length': '32', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'recurly_account'", 'null': 'True', 'to': "orm['auth.User']"})
        },
        'django_recurly.payment': {
            'Meta': {'ordering': "['-id']", 'object_name': 'Payment'},
            'account': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['django_recurly.Account']", 'null': 'True', 'blank': 'True'}),
            'action': ('django.db.models.fields.CharField', [], {'max_length': '10'}),
            'amount_in_cents': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'date': ('timezones.fields.LocalizedDateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'invoice_id': ('django.db.models.fields.CharField', [], {'max_length': '40', 'null': 'True', 'blank': 'True'}),
            'message': ('django.db.models.fields.CharField', [], {'max_length': '250'}),
            'status': ('django.db.models.fields.CharField', [], {'max_length': '10'}),
            'transaction_id': ('django.db.models.fields.CharField', [], {'max_length': '40'})
        },
        'django_recurly.subscription': {
            'Meta': {'ordering': "['-id']", 'object_name': 'Subscription'},
            'account': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['django_recurly.Account']"}),
            'activated_at': ('timezones.fields.LocalizedDateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'canceled_at': ('timezones.fields.LocalizedDateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'current_period_ends_at': ('timezones.fields.LocalizedDateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'current_period_started_at': ('timezones.fields.LocalizedDateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'expires_at': ('timezones.fields.LocalizedDateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'plan_code': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'plan_version': ('django.db.models.fields.IntegerField', [], {'default': '1'}),
            'quantity': ('django.db.models.fields.IntegerField', [], {'default': '1'}),
            'state': ('django.db.models.fields.CharField', [], {'default': "'active'", 'max_length': '20'}),
            'super_subscription': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'total_amount_in_cents': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'trial_ends_at': ('timezones.fields.LocalizedDateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'trial_started_at': ('timezones.fields.LocalizedDateTimeField', [], {'null': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['django_recurly']



"""Template tags for working work Recurly's API"""

from django import template
from django.template import Library, Node, Variable, loader
from django.template.context import Context

from django_recurly.helpers.api import get_change_plan_form

register = template.Library()

@register.simple_tag
def change_plan_form(plan_code):
    return get_change_plan_form(plan_code)




# -*- coding: utf-8 -*-

import os.path

# These should NOT be valid credentials
RECURLY_USERNAME = "dummy_username"
RECURLY_PASSWORD = "dummy_password"
RECURLY_SUBDOMAIN = "dummy_subdomain"

RECURLY_PLANS = ("micro", "small", "medium", "large")

# The username & password used to authorise Recurly's 
# postback notifications. In the format "username:password"
RECURLY_HTTP_AUTHENTICATION = "1234567890abcdefghijklmnop:abcdefghijklmnop0987654321"

# For the love of all things holy, please keep this set to a sensible 
# (i.e. unchanging / non daylight saving) timezone. This determines the 
# timezone  in which django_recurly stores dates in the DB, so if you 
# change this down the road then all your subscription dates will skew.
# The sane thing to do here is to keep this as UTC and handle the 
# timezone conversion in your display logic.
TIME_ZONE = "UTC"

PROJECT_ROOT = os.path.abspath(os.path.dirname(__file__))

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": "%s/dev.db" % PROJECT_ROOT,
        "USER": "",
        "PASSWORD": "",
        "HOST": "",
        "PORT": "",
    }
}

INSTALLED_APPS = (
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django_recurly",
)

SITE_ID = 1

SECRET_KEY = "r0eCEzRzC8s2u8h3L!*tr9v,kz!gm:"

ROOT_URLCONF = "django_recurly.urls"

# local_settings.py can be used to override environment-specific settings
# like database and email that differ between development and production.
try:
    from local_settings import *
except ImportError:
    pass


# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'Account'
        db.create_table('django_recurly_account', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('account_code', self.gf('django.db.models.fields.CharField')(unique=True, max_length=32)),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(related_name='recurly_account', to=orm['auth.User'])),
            ('created_at', self.gf('timezones.fields.LocalizedDateTimeField')(default=datetime.datetime(2011, 10, 4, 10, 8, 10, 230744))),
            ('email', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('first_name', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('last_name', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('company_name', self.gf('django.db.models.fields.CharField')(max_length=100, null=True, blank=True)),
            ('canceled', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('hosted_login_token', self.gf('django.db.models.fields.CharField')(max_length=32, null=True, blank=True)),
        ))
        db.send_create_signal('django_recurly', ['Account'])

        # Adding model 'Subscription'
        db.create_table('django_recurly_subscription', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('account', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['django_recurly.Account'])),
            ('plan_code', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('plan_version', self.gf('django.db.models.fields.IntegerField')(default=1)),
            ('state', self.gf('django.db.models.fields.CharField')(default='active', max_length=20)),
            ('quantity', self.gf('django.db.models.fields.IntegerField')(default=1)),
            ('total_amount_in_cents', self.gf('django.db.models.fields.DecimalField')(null=True, max_digits=8, decimal_places=2, blank=True)),
            ('activated_at', self.gf('timezones.fields.LocalizedDateTimeField')(null=True, blank=True)),
            ('canceled_at', self.gf('timezones.fields.LocalizedDateTimeField')(null=True, blank=True)),
            ('expires_at', self.gf('timezones.fields.LocalizedDateTimeField')(null=True, blank=True)),
            ('current_period_started_at', self.gf('timezones.fields.LocalizedDateTimeField')(null=True, blank=True)),
            ('current_period_ends_at', self.gf('timezones.fields.LocalizedDateTimeField')(null=True, blank=True)),
            ('trial_started_at', self.gf('timezones.fields.LocalizedDateTimeField')(null=True, blank=True)),
            ('trial_ends_at', self.gf('timezones.fields.LocalizedDateTimeField')(null=True, blank=True)),
            ('super_subscription', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal('django_recurly', ['Subscription'])


    def backwards(self, orm):
        
        # Deleting model 'Account'
        db.delete_table('django_recurly_account')

        # Deleting model 'Subscription'
        db.delete_table('django_recurly_subscription')


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'django_recurly.account': {
            'Meta': {'ordering': "['-id']", 'object_name': 'Account'},
            'account_code': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '32'}),
            'canceled': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'company_name': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'created_at': ('timezones.fields.LocalizedDateTimeField', [], {'default': 'datetime.datetime(2011, 10, 4, 10, 8, 10, 230744)'}),
            'email': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'hosted_login_token': ('django.db.models.fields.CharField', [], {'max_length': '32', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'recurly_account'", 'to': "orm['auth.User']"})
        },
        'django_recurly.subscription': {
            'Meta': {'ordering': "['-id']", 'object_name': 'Subscription'},
            'account': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['django_recurly.Account']"}),
            'activated_at': ('timezones.fields.LocalizedDateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'canceled_at': ('timezones.fields.LocalizedDateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'current_period_ends_at': ('timezones.fields.LocalizedDateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'current_period_started_at': ('timezones.fields.LocalizedDateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'expires_at': ('timezones.fields.LocalizedDateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'plan_code': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'plan_version': ('django.db.models.fields.IntegerField', [], {'default': '1'}),
            'quantity': ('django.db.models.fields.IntegerField', [], {'default': '1'}),
            'state': ('django.db.models.fields.CharField', [], {'default': "'active'", 'max_length': '20'}),
            'super_subscription': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'total_amount_in_cents': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '8', 'decimal_places': '2', 'blank': 'True'}),
            'trial_ends_at': ('timezones.fields.LocalizedDateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'trial_started_at': ('timezones.fields.LocalizedDateTimeField', [], {'null': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['django_recurly']



from django.conf import settings

def get_client():
    from recurly import Recurly
    
    return Recurly(
        username=settings.RECURLY_USERNAME,
        password=settings.RECURLY_PASSWORD,
        subdomain=settings.RECURLY_SUBDOMAIN,
    )



#!/usr/bin/env python

from setuptools import setup, find_packages
from django_recurly import __version__

setup(
    name='django-recurly',
    license='BSD',
    packages=find_packages(),
    version=__version__,
    url="https://github.com/adamcharnock/django-recurly",
    author="Adam Charnock",
    author_email="adam@playnice.ly",
    description="Django integration for Recurly, a subscription billing service.",
    
    install_requires=[
        "iso8601",
        "django-timezones",
    ]
)



import base64

from django.shortcuts import render_to_response
from django.template import RequestContext
from django.http import HttpResponse, HttpResponseBadRequest, HttpResponseForbidden
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_POST
from django.conf import settings
from django.utils.crypto import constant_time_compare

from django_recurly.client import get_client
from django_recurly.utils import safe_redirect
from django_recurly.models import Account, Subscription
from django_recurly.decorators import recurly_basic_authentication

from django_recurly import signals

@csrf_exempt
@recurly_basic_authentication
@require_POST
def push_notifications(request):
    
    client = get_client()
    
    name = client.parse_notification(request.raw_post_data)
    
    try:
        signal = getattr(signals, name)
    except AttributeError:
        return HttpResponseBadRequest("Invalid notification name.")
    
    signal.send(sender=client, data=client.response)
    
    return HttpResponse()

@require_POST
def change_plan(request):
    new_plan = request.POST.get("plan_code")
    
    subscription = Account.get_current(request.user).get_current_subscription()
    subscription.change_plan(new_plan)
    
    redirect_to = request.POST.get("redirect_to", None)
    
    return safe_redirect(redirect_to)

def account(request):
    account = Account.get_current(request.user)
    subscription = account.get_current_subscription()
    
    c = {
        "account": account,
        "subscription": subscription,
        "plans": settings.RECURLY_PLANS
    }
    
    return render_to_response("django_recurly/account.html", c, RequestContext(request))



# -*- coding: utf-8 -*-

from django.test import TestCase

from django_recurly.helpers import hosted
from django_recurly.tests.base import BaseTest
from django_recurly.models import *


class GetSubscribeUrlTest(BaseTest):
    
    def test_encoding(self):
        expected = "https://dummy_subdomain.recurly.com/subscribe/myplan/mya%C3%A7%C3%A7o+unt/ad%2Fa%C3%A5m?quantity=2&first_name=%C3%85dam&last_name=%C3%87harnock&email=adam@test.com"
        
        self.user.username = u"ad/aåm"
        self.user.first_name = u"Ådam"
        self.user.last_name = u"Çharnock"
        self.user.email = u"adam@test.com"
        
        self.assertEqual(hosted.get_subscribe_url(
            plan_code=u"myplan",
            user=self.user,
            quantity=2,
            account_code=u"myaçço unt"
        ), expected)
    



import unittest
import datetime
import pytz

from django.test import TestCase

from django_recurly.tests.base import BaseTest
from django_recurly.models import *

class AccountModelTest(BaseTest):
    
    def test_handle_notification_creating(self):
        data = self.parse_xml(self.push_notifications["new_subscription_notification-ok"])
        account, subscription = Account.handle_notification(data)
        
        self.assertEqual(Account.objects.count(), 1)
        self.assertEqual(Subscription.objects.count(), 1)
        
        # Lets be through here
        self.assertEqual(account.user.username, "verena")
        self.assertEqual(account.first_name, "Verena")
        self.assertEqual(account.company_name, "Company, Inc.")
        self.assertEqual(account.email, "verena@test.com")
        self.assertEqual(account.account_code, "verena@test.com")
        
        subscription = account.get_current_subscription()
        self.assertEqual(subscription.plan_code, "bronze")
        self.assertEqual(subscription.plan_version, 2)
        self.assertEqual(subscription.state, "active")
        self.assertEqual(subscription.quantity, 2)
        self.assertEqual(subscription.total_amount_in_cents, 2000)
        self.assertEqual(subscription.activated_at, datetime.datetime(2009, 11, 22, 21, 10, 38)) # Phew, its in UTC now :)
        
        self.assertSignal("account_opened")
        self.assertNoSignal("account_closed")
    
    def test_handle_notification_updating_cancelled(self):
        data = self.parse_xml(self.push_notifications["new_subscription_notification-ok"])
        Account.handle_notification(data)
        
        self.resetSignals()
        
        data = self.parse_xml(self.push_notifications["canceled_subscription_notification-ok"])
        account, subscription = Account.handle_notification(data)
        
        # Lets be through here
        self.assertEqual(account.user.username, "verena")
        self.assertEqual(account.first_name, "Jane")
        self.assertEqual(account.last_name, "Doe")
        self.assertEqual(account.company_name, None)
        self.assertEqual(account.email, "janedoe@gmail.com")
        self.assertEqual(account.account_code, "verena@test.com")
        
        subscription = account.get_current_subscription()
        self.assertEqual(subscription.plan_code, "1dpt")
        self.assertEqual(subscription.plan_version, 2)
        self.assertEqual(subscription.state, "canceled")
        self.assertEqual(subscription.quantity, 1)
        self.assertEqual(subscription.total_amount_in_cents, 200)
        
        # Account was 'cancelled', but is still technically open until is expires
        self.assertNoSignal("account_opened")
        self.assertNoSignal("account_closed")
    
    def test_handle_notification_updating_expired(self):
        data = self.parse_xml(self.push_notifications["new_subscription_notification-ok"])
        Account.handle_notification(data)
        
        self.resetSignals()
        
        data = self.parse_xml(self.push_notifications["expired_subscription_notification-ok"])
        account, subscription = Account.handle_notification(data)
        
        # Lets be through here
        self.assertEqual(account.user.username, "verena")
        self.assertEqual(account.first_name, "Jane")
        self.assertEqual(account.last_name, "Doe")
        self.assertEqual(account.company_name, None)
        self.assertEqual(account.email, "janedoe@gmail.com")
        self.assertEqual(account.account_code, "verena@test.com")
        
        subscription = account.get_current_subscription()
        self.assertEqual(subscription, None)
        
        subscription = account.get_subscriptions().latest()
        self.assertEqual(subscription.plan_code, "1dpt")
        self.assertEqual(subscription.plan_version, 2)
        self.assertEqual(subscription.state, "expired")
        self.assertEqual(subscription.quantity, 1)
        self.assertEqual(subscription.total_amount_in_cents, 200)
        
        self.assertNoSignal("account_opened")
        self.assertSignal("account_closed")
    
    def test_handle_notification_updating_expired_real(self):
        # Straight in with no prior account
        data = self.parse_xml(self.push_notifications["expired_subscription_notification-real"])
        account, subscription = Account.handle_notification(data)
        
        # Lets be through here
        self.assertEqual(account.user.username, "verena")
        self.assertEqual(account.first_name, "Adam")
        self.assertEqual(account.last_name, "Charnock")
        self.assertEqual(account.company_name, None)
        self.assertEqual(account.email, "adam@continuous.io")
        self.assertEqual(account.account_code, "vKWanguTh5KcZniN0yZeFbjD8xmFfVGT")
        
        subscription = account.get_current_subscription()
        self.assertEqual(subscription, None)
        
        subscription = account.get_subscriptions().latest()
        self.assertEqual(subscription.plan_code, "micro")
        self.assertEqual(subscription.plan_version, 1)
        self.assertEqual(subscription.state, "expired")
        self.assertEqual(subscription.quantity, 1)
        self.assertEqual(subscription.total_amount_in_cents, 700)
        
        self.assertEqual(subscription.activated_at, datetime.datetime(2011, 9, 14, 19, 14, 14)) # Phew, its in UTC now :)
        
        # The subscription was created as 'expired' right away, so no signals
        self.assertNoSignal("account_opened")
        self.assertNoSignal("account_closed")
    
    def test_handle_notification_new_after_expired(self):
        data = self.parse_xml(self.push_notifications["new_subscription_notification-ok"])
        Account.handle_notification(data)
        
        data = self.parse_xml(self.push_notifications["expired_subscription_notification-ok"])
        Account.handle_notification(data)
        
        self.resetSignals()
        
        data = self.parse_xml(self.push_notifications["new_subscription_notification-ok"])
        Account.handle_notification(data)
        
        self.assertEqual(Account.objects.count(), 1)
        # We should now have the old expired subscription, plus the fresh new one
        self.assertEqual(Subscription.objects.count(), 2)
        self.assertEqual(Subscription.objects.latest().state, "active")
        
        self.assertSignal("account_opened")
        self.assertNoSignal("account_closed")
    
    def test_get_current(self):
        data = self.parse_xml(self.push_notifications["new_subscription_notification-ok"])
        account, subscription = Account.handle_notification(data)
        
        account = Account.get_current(self.user)
        self.assertEqual(account.user.username, "verena")

class SubscriptionModelTest(BaseTest):
    
    def test_is_current(self):
        data = self.parse_xml(self.push_notifications["new_subscription_notification-ok"])
        account, subscription = Account.handle_notification(data)
        
        self.assertTrue(Subscription.objects.latest().is_current())
        
        data = self.parse_xml(self.push_notifications["expired_subscription_notification-ok"])
        account, subscription = Account.handle_notification(data)
        
        self.assertFalse(Subscription.objects.latest().is_current())
        
        subscription.super_subscription = True
        subscription.save()
        
        self.assertTrue(Subscription.objects.latest().is_current())
    
    def test_is_trial(self):
        data = self.parse_xml(self.push_notifications["new_subscription_notification-ok"])
        account, subscription = Account.handle_notification(data)
        
        self.assertFalse(subscription.is_trial())
        
        subscription.trial_ends_at = datetime.datetime.now(tz=pytz.utc) + datetime.timedelta(days=1)
        subscription.save()
        
        self.assertTrue(subscription.is_trial())
    

class PaymentModelTest(BaseTest):
    
    def test_handle_payment_successful(self):
        data = self.parse_xml(self.push_notifications["new_subscription_notification-ok"])
        account, subscription = Account.handle_notification(data)
        
        data = self.parse_xml(self.push_notifications["successful_payment_notification-ok"])
        payment = Payment.handle_notification(data)
        
        self.assertEqual(Payment.objects.count(), 1)
        
        payment = Payment.objects.all().latest()
        self.assertEqual(payment.transaction_id, "a5143c1d3a6f4a8287d0e2cc1d4c0427")
        self.assertEqual(payment.invoice_id, "ffc64d71d4b5404e93f13aac9c63bxxx")
        self.assertEqual(payment.action, "purchase")
        
        self.assertEqual(payment.date, datetime.datetime(2009, 11, 22, 21, 10, 38)) # Phew, its in UTC now :)
        self.assertEqual(payment.amount_in_cents, 1000)
        self.assertEqual(payment.status, "success")
        self.assertEqual(payment.message, "Bogus Gateway: Forced success")
    
    def test_handle_refund(self):
        data = self.parse_xml(self.push_notifications["new_subscription_notification-ok"])
        account, subscription = Account.handle_notification(data)
        
        data = self.parse_xml(self.push_notifications["successful_refund_notification-ok"])
        payment = Payment.handle_notification(data)
        
        self.assertEqual(Payment.objects.count(), 1)
        
        payment = Payment.objects.all().latest()
        self.assertEqual(payment.transaction_id, "2c7a2e30547e49869efd4e8a44b2be34")
        self.assertEqual(payment.invoice_id, "ffc64d71d4b5404e93f13aac9c63b007")
        self.assertEqual(payment.action, "credit")
        
        self.assertEqual(payment.amount_in_cents, 235)
        self.assertEqual(payment.status, "success")
        self.assertEqual(payment.message, "Bogus Gateway: Forced success")
    
    def test_handle_void(self):
        data = self.parse_xml(self.push_notifications["new_subscription_notification-ok"])
        account, subscription = Account.handle_notification(data)
        
        data = self.parse_xml(self.push_notifications["void_payment_notification-ok"])
        payment = Payment.handle_notification(data)
        
        self.assertEqual(Payment.objects.count(), 1)
        
        payment = Payment.objects.all().latest()
        self.assertEqual(payment.transaction_id, "4997ace0f57341adb3e857f9f7d15de8")
        self.assertEqual(payment.invoice_id, "ffc64d71d4b5404e93f13aac9c63b007")
        self.assertEqual(payment.action, "purchase")
        
        self.assertEqual(payment.amount_in_cents, 235)
        self.assertEqual(payment.status, "void")
        self.assertEqual(payment.message, "Test Gateway: Successful test transaction")
    
    def test_handle_failed(self):
        data = self.parse_xml(self.push_notifications["new_subscription_notification-ok"])
        account, subscription = Account.handle_notification(data)
        
        data = self.parse_xml(self.push_notifications["failed_payment_notification-ok"])
        payment = Payment.handle_notification(data)
        
        self.assertEqual(Payment.objects.count(), 1)
        
        payment = Payment.objects.all().latest()
        self.assertEqual(payment.transaction_id, "a5143c1d3a6f4a8287d0e2cc1d4c0427")
        self.assertEqual(payment.invoice_id, None)
        self.assertEqual(payment.action, "purchase")
        
        self.assertEqual(payment.amount_in_cents, 1000)
        self.assertEqual(payment.status, "declined")
        self.assertEqual(payment.message, "This transaction has been declined")
    












"""Helpers for working with Recurly's Hosted Payment Pages"""

import urllib

from django.conf import settings
from django.core.urlresolvers import reverse
from django.utils.http import urlquote_plus
from django.utils.encoding import iri_to_uri
from django.http import HttpResponseRedirect

from django_recurly.client import get_client
from django_recurly.utilities import random_string

SUBSCRIBE_URL = "https://%(subdomain)s.recurly.com/subscribe/%(plan_code)s/%(account_code)s/%(username)s?quantity=%(quantity)s&first_name=%(first_name)s&last_name=%(last_name)s&email=%(email)s"

def get_subscribe_url(user, plan_code, quantity=1, account_code=None):
    """Generate the URL for the hosted payment page
    
    Redirect the user to this URL to send them off to Recurly.
    
    You should not normally need to specify 'account_code'
    """
    
    params = {
        "subdomain": get_client().subdomain,
        "plan_code": urlquote_plus(plan_code),
        "account_code": urlquote_plus(account_code) if account_code else random_string(length=32),
        "username": urlquote_plus(user.username),
        
        "quantity": quantity,
        "first_name": getattr(user, "first_name", ""),
        "last_name": getattr(user, "last_name", ""),
        "email": getattr(user, "email", ""),
    }
    
    subscribe_url = iri_to_uri(SUBSCRIBE_URL % params)
    
    return subscribe_url

class SubscribeRedirect(HttpResponseRedirect):
    """A simple HTTP response class which can be returned 
    from views in order to send users to the hosted payment page"""
    
    status_code = 302

    def __init__(self, plan_code, user, quantity=1, account_code=None):
        super(HttpResponseRedirect, self).__init__()
        
        self['Location'] = get_subscribe_url(plan_code, user, quantity, account_code)
    



